/** @file inc_switch_setup
    Functions for switch setups. Most only used on
    mod load. Get/Set individual switches is done
    via prc_inc_switch.
    DO NOT include this file if you just want to
    use Set/GetPRCSwitch as it's not needed.
*/

//////////////////////////////////////////////////
/*             Function prototypes              */
//////////////////////////////////////////////////

/**
 * Controls which itemproperties can be added to
 * the samurai weapon.
 * @see PRC_SAMURAI_BAN_ in prc_inc_switch and the
 * function difinition for more info.
 */
void DoSamuraiBanDefaults();

/**
 * Sets the epic spell switches to their default values.
 *
 * If PRC_EPIC_INGORE_DEFAULTS is set, this does nothing.
 */
void DoEpicSpellDefaults();

/**
 * Sets the file end markers to their default values.
 *
 * If FILE_END_MANUAL is set, this does nothing.
 */
void SetDefaultFileEnds();

/**
 * This creates an array of all switch names on a waypoint
 * It is used for the switch setting convo to loop over switches easily
 */
void CreateSwitchNameArray();

//////////////////////////////////////////////////
/* Include section                              */
//////////////////////////////////////////////////

#include "inc_array" // Needs direct include instead of inc_utility
#include "prc_inc_switch"

//////////////////////////////////////////////////
/*             Function definitions             */
//////////////////////////////////////////////////

void DoSamuraiBanDefaults()
{
    if(GetPRCSwitch(PRC_SAMURAI_DISABLE_DEFAULT_BAN))
        return;
    //remove all penalty iprops
    SetPRCSwitch(PRC_SAMURAI_BAN_+"10_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"21_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"24_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"27_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"28_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"29_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"47_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"49_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"50_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"60_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"62_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"63_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"64_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"65_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"66_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"81_*_*_*", TRUE);
    //PRCs restrictions
    SetPRCSwitch(PRC_SAMURAI_BAN_+"86_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"87_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"88_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"89_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"90_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"91_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"120_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"121_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"122_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"123_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"124_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"125_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"126_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"127_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"134_*_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"150_*_*_*", TRUE);
    //only allow elemental damages 6,7,9,10,13
    //damage
    SetPRCSwitch(PRC_SAMURAI_BAN_+"16_5_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"16_8_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"16_11_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"16_12_*_*", TRUE);
    //damage vs race
    SetPRCSwitch(PRC_SAMURAI_BAN_+"17_*_5_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"17_*_8_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"17_*_11_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"17_*_12_*", TRUE);
    //damage vs alignment
    SetPRCSwitch(PRC_SAMURAI_BAN_+"18_*_5_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"18_*_8_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"18_*_11_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"18_*_12_*", TRUE);
    //damage vs specific alignment
    SetPRCSwitch(PRC_SAMURAI_BAN_+"19_*_5_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"19_*_8_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"19_*_11_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"19_*_12_*", TRUE);
    //damage immunity
    SetPRCSwitch(PRC_SAMURAI_BAN_+"20_5_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"20_8_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"20_11_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"20_12_*_*", TRUE);
    //damage resist
    SetPRCSwitch(PRC_SAMURAI_BAN_+"20_5_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"20_8_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"20_11_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"20_12_*_*", TRUE);
    //slays
    SetPRCSwitch(PRC_SAMURAI_BAN_+"48_21_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"48_22_*_*", TRUE);
    SetPRCSwitch(PRC_SAMURAI_BAN_+"48_23_*_*", TRUE);
    //vorpal
    SetPRCSwitch(PRC_SAMURAI_BAN_+"48_24_*_*", TRUE);
}

void DoEpicSpellDefaults()
{
    if(GetPRCSwitch(PRC_EPIC_INGORE_DEFAULTS))
        return;
    SetPRCSwitch(PRC_EPIC_XP_COSTS, TRUE);
    SetPRCSwitch(PRC_EPIC_BACKLASH_DAMAGE, TRUE);
    SetPRCSwitch(PRC_EPIC_FOCI_ADJUST_DC, TRUE);
    SetPRCSwitch(PRC_EPIC_GOLD_MULTIPLIER, 9000);
    SetPRCSwitch(PRC_EPIC_XP_FRACTION, 25);
    SetPRCSwitch(PRC_EPIC_FAILURE_FRACTION_GOLD, 2);
    SetPRCSwitch(PRC_EPIC_BOOK_DESTRUCTION, 50);
}

void SetDefaultFileEnds()
{
    // Overridden values: (TMI prevention)
    //
    // feat.2da - 390
    // spells.2da - 539

    //START AUTO-GENERATED FILEENDS
    SetPRCSwitch("PRC_FILE_END_actions", 43);
    SetPRCSwitch("PRC_FILE_END_ambientmusic", 95);
    SetPRCSwitch("PRC_FILE_END_ambientsound", 113);
    SetPRCSwitch("PRC_FILE_END_ammunitiontypes", 35);
    SetPRCSwitch("PRC_FILE_END_appearance", 870);
    SetPRCSwitch("PRC_FILE_END_appearancesndset", 31);
    SetPRCSwitch("PRC_FILE_END_areaeffects", 2);
    SetPRCSwitch("PRC_FILE_END_armor", 8);
    SetPRCSwitch("PRC_FILE_END_armorparts", 0);
    SetPRCSwitch("PRC_FILE_END_armourtypes", 42);
    SetPRCSwitch("PRC_FILE_END_baseitems", 204);
    SetPRCSwitch("PRC_FILE_END_bodybag", 6);
    SetPRCSwitch("PRC_FILE_END_caarmorclass", 7);
    SetPRCSwitch("PRC_FILE_END_capart", 18);
    SetPRCSwitch("PRC_FILE_END_categories", 22);
    SetPRCSwitch("PRC_FILE_END_catype", 4);
    SetPRCSwitch("PRC_FILE_END_chargenclothes", 15);
    SetPRCSwitch("PRC_FILE_END_classes", 254);
    SetPRCSwitch("PRC_FILE_END_cloakmodel", 16);
    SetPRCSwitch("PRC_FILE_END_cls_atk_1", 59);
    SetPRCSwitch("PRC_FILE_END_cls_atk_2", 59);
    SetPRCSwitch("PRC_FILE_END_cls_atk_3", 59);
    SetPRCSwitch("PRC_FILE_END_cls_atk_4", 59);
    SetPRCSwitch("PRC_FILE_END_cls_atk_adst", 59);
    SetPRCSwitch("PRC_FILE_END_cls_invkn_dfa", 59);
    SetPRCSwitch("PRC_FILE_END_cls_invkn_warlok", 59);
    SetPRCSwitch("PRC_FILE_END_cls_inv_dfa", 43);
    SetPRCSwitch("PRC_FILE_END_cls_inv_warlok", 112);
    SetPRCSwitch("PRC_FILE_END_cls_ivcr_dfa", 22);
    SetPRCSwitch("PRC_FILE_END_cls_ivcr_warlok", 93);
    SetPRCSwitch("PRC_FILE_END_cls_move_crusdr", 211);
    SetPRCSwitch("PRC_FILE_END_cls_move_swdsge", 211);
    SetPRCSwitch("PRC_FILE_END_cls_move_warbld", 211);
    SetPRCSwitch("PRC_FILE_END_cls_mvcr_crusdr", 207);
    SetPRCSwitch("PRC_FILE_END_cls_mvcr_swdsge", 207);
    SetPRCSwitch("PRC_FILE_END_cls_mvcr_warbld", 207);
    SetPRCSwitch("PRC_FILE_END_cls_mvkn_crusdr", 59);
    SetPRCSwitch("PRC_FILE_END_cls_mvkn_swdsge", 59);
    SetPRCSwitch("PRC_FILE_END_cls_mvkn_warbld", 59);
    SetPRCSwitch("PRC_FILE_END_cls_psbk_foz", 59);
    SetPRCSwitch("PRC_FILE_END_cls_psbk_psion", 59);
    SetPRCSwitch("PRC_FILE_END_cls_psbk_psywar", 59);
    SetPRCSwitch("PRC_FILE_END_cls_psbk_warmnd", 59);
    SetPRCSwitch("PRC_FILE_END_cls_psbk_wilder", 59);
    SetPRCSwitch("PRC_FILE_END_cls_psicr_foz", 154);
    SetPRCSwitch("PRC_FILE_END_cls_psicr_psion", 196);
    SetPRCSwitch("PRC_FILE_END_cls_psicr_psywar", 169);
    SetPRCSwitch("PRC_FILE_END_cls_psicr_warmnd", 154);
    SetPRCSwitch("PRC_FILE_END_cls_psicr_wilder", 196);
    SetPRCSwitch("PRC_FILE_END_cls_psipw_foz", 226);
    SetPRCSwitch("PRC_FILE_END_cls_psipw_psion", 285);
    SetPRCSwitch("PRC_FILE_END_cls_psipw_psywar", 246);
    SetPRCSwitch("PRC_FILE_END_cls_psipw_warmnd", 226);
    SetPRCSwitch("PRC_FILE_END_cls_psipw_wilder", 285);
    SetPRCSwitch("PRC_FILE_END_cls_savthr_barb", 59);
    SetPRCSwitch("PRC_FILE_END_cls_savthr_bard", 60);
    SetPRCSwitch("PRC_FILE_END_cls_savthr_cler", 59);
    SetPRCSwitch("PRC_FILE_END_cls_savthr_cons", 59);
    SetPRCSwitch("PRC_FILE_END_cls_savthr_dru", 59);
    SetPRCSwitch("PRC_FILE_END_cls_savthr_fight", 59);
    SetPRCSwitch("PRC_FILE_END_cls_savthr_lich", 59);
    SetPRCSwitch("PRC_FILE_END_cls_savthr_monk", 59);
    SetPRCSwitch("PRC_FILE_END_cls_savthr_pal", 59);
    SetPRCSwitch("PRC_FILE_END_cls_savthr_rang", 59);
    SetPRCSwitch("PRC_FILE_END_cls_savthr_rog", 59);
    SetPRCSwitch("PRC_FILE_END_cls_savthr_sorc", 59);
    SetPRCSwitch("PRC_FILE_END_cls_savthr_wild", 59);
    SetPRCSwitch("PRC_FILE_END_cls_savthr_wiz", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_antipl", 42);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_archv", 502);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_asasin", 39);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_bard", 144);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_beguil", 142);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_blkgrd", 47);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_dnecro", 137);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_duskbl", 69);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_favsol", 290);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_harper", 35);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_healer", 77);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_hexbl", 73);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_justww", 34);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_kchal", 48);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_kotmc", 26);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_myst", 290);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_ocu", 209);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_schord", 244);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_shaman", 205);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_sod", 33);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_sohei", 57);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_sol", 38);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_sorc", 430);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_suel", 117);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_templ", 112);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_tfshad", 27);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_vassal", 31);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_vigil", 41);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_witch", 252);
    SetPRCSwitch("PRC_FILE_END_cls_spcr_wrmage", 135);
    SetPRCSwitch("PRC_FILE_END_cls_spell_antipl", 115);
    SetPRCSwitch("PRC_FILE_END_cls_spell_archv", 2720);
    SetPRCSwitch("PRC_FILE_END_cls_spell_asasin", 52);
    SetPRCSwitch("PRC_FILE_END_cls_spell_bard", 169);
    SetPRCSwitch("PRC_FILE_END_cls_spell_beguil", 119);
    SetPRCSwitch("PRC_FILE_END_cls_spell_blkgrd", 163);
    SetPRCSwitch("PRC_FILE_END_cls_spell_dnecro", 134);
    SetPRCSwitch("PRC_FILE_END_cls_spell_duskbl", 84);
    SetPRCSwitch("PRC_FILE_END_cls_spell_favsol", 363);
    SetPRCSwitch("PRC_FILE_END_cls_spell_harper", 21);
    SetPRCSwitch("PRC_FILE_END_cls_spell_healer", 271);
    SetPRCSwitch("PRC_FILE_END_cls_spell_hexbl", 79);
    SetPRCSwitch("PRC_FILE_END_cls_spell_justww", 26);
    SetPRCSwitch("PRC_FILE_END_cls_spell_kchal", 137);
    SetPRCSwitch("PRC_FILE_END_cls_spell_kotmc", 70);
    SetPRCSwitch("PRC_FILE_END_cls_spell_myst", 363);
    SetPRCSwitch("PRC_FILE_END_cls_spell_ocu", 905);
    SetPRCSwitch("PRC_FILE_END_cls_spell_schord", 308);
    SetPRCSwitch("PRC_FILE_END_cls_spell_shaman", 667);
    SetPRCSwitch("PRC_FILE_END_cls_spell_sod", 110);
    SetPRCSwitch("PRC_FILE_END_cls_spell_sohei", 131);
    SetPRCSwitch("PRC_FILE_END_cls_spell_sol", 114);
    SetPRCSwitch("PRC_FILE_END_cls_spell_sorc", 541);
    SetPRCSwitch("PRC_FILE_END_cls_spell_suel", 160);
    SetPRCSwitch("PRC_FILE_END_cls_spell_templ", 95);
    SetPRCSwitch("PRC_FILE_END_cls_spell_tfshad", 70);
    SetPRCSwitch("PRC_FILE_END_cls_spell_vassal", 104);
    SetPRCSwitch("PRC_FILE_END_cls_spell_vigil", 75);
    SetPRCSwitch("PRC_FILE_END_cls_spell_witch", 193);
    SetPRCSwitch("PRC_FILE_END_cls_spell_wrmage", 147);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_antipl", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_archv", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_asasin", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_bard", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_beguil", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_blkgrd", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_cler", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_dnecro", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_dru", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_duskbl", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_favsol", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_harper", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_healer", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_hexbl", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_justww", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_kchal", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_kotmc", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_myst", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_ocu", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_pal", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_rang", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_schord", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_shaman", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_sod", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_sohei", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_sol", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_sorc", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_suel", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_templ", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_tfshad", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_vassal", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_vigil", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_witch", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_wiz", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spgn_wrmage", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spkn_asasin", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spkn_bard", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spkn_beguil", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spkn_dnecro", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spkn_duskbl", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spkn_favsol", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spkn_harper", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spkn_hexbl", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spkn_justww", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spkn_myst", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spkn_schord", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spkn_sorc", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spkn_suel", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spkn_templ", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spkn_witch", 59);
    SetPRCSwitch("PRC_FILE_END_cls_spkn_wrmage", 59);
    SetPRCSwitch("PRC_FILE_END_cls_true_known", 39);
    SetPRCSwitch("PRC_FILE_END_cls_true_maxlvl", 39);
    SetPRCSwitch("PRC_FILE_END_cls_true_utter", 166);
    SetPRCSwitch("PRC_FILE_END_colours", 175);
    SetPRCSwitch("PRC_FILE_END_combatmodes", 3);
    SetPRCSwitch("PRC_FILE_END_craft_armour", 63);
    SetPRCSwitch("PRC_FILE_END_craft_golem", 40);
    SetPRCSwitch("PRC_FILE_END_craft_ring", 41);
    SetPRCSwitch("PRC_FILE_END_craft_weapon", 45);
    SetPRCSwitch("PRC_FILE_END_craft_wondrous", 115);
    SetPRCSwitch("PRC_FILE_END_creaturesize", 5);
    SetPRCSwitch("PRC_FILE_END_creaturespeed", 8);
    SetPRCSwitch("PRC_FILE_END_crtemplates", 10);
    SetPRCSwitch("PRC_FILE_END_cursors", 10);
    SetPRCSwitch("PRC_FILE_END_damagehitvisual", 11);
    SetPRCSwitch("PRC_FILE_END_damagelevels", 5);
    SetPRCSwitch("PRC_FILE_END_defaultacsounds", 8);
    SetPRCSwitch("PRC_FILE_END_des_blumburg", 17);
    SetPRCSwitch("PRC_FILE_END_des_conf_treas", 1);
    SetPRCSwitch("PRC_FILE_END_des_crft_amat", 1);
    SetPRCSwitch("PRC_FILE_END_des_crft_aparts", 17);
    SetPRCSwitch("PRC_FILE_END_des_crft_appear", 53);
    SetPRCSwitch("PRC_FILE_END_des_crft_armor", 41);
    SetPRCSwitch("PRC_FILE_END_des_crft_bmat", 13);
    SetPRCSwitch("PRC_FILE_END_des_crft_drop", 476);
    SetPRCSwitch("PRC_FILE_END_des_crft_mat", 2);
    SetPRCSwitch("PRC_FILE_END_des_crft_poison", 100);
    SetPRCSwitch("PRC_FILE_END_des_crft_props", 27);
    SetPRCSwitch("PRC_FILE_END_des_crft_scroll", 3999);
    SetPRCSwitch("PRC_FILE_END_des_crft_spells", 19348);
    SetPRCSwitch("PRC_FILE_END_des_crft_weapon", 29);
    SetPRCSwitch("PRC_FILE_END_des_cutconvdur", 26);
    SetPRCSwitch("PRC_FILE_END_des_feat2item", 1000);
    SetPRCSwitch("PRC_FILE_END_des_matcomp", 510);
    SetPRCSwitch("PRC_FILE_END_des_mechupgrades", 6);
    SetPRCSwitch("PRC_FILE_END_des_pcstart_arm", 1);
    SetPRCSwitch("PRC_FILE_END_des_pcstart_weap", 1);
    SetPRCSwitch("PRC_FILE_END_des_prayer", 9);
    SetPRCSwitch("PRC_FILE_END_des_restsystem", 21);
    SetPRCSwitch("PRC_FILE_END_des_treas_ammo", 28);
    SetPRCSwitch("PRC_FILE_END_des_treas_disp", 417);
    SetPRCSwitch("PRC_FILE_END_des_treas_enh", 60);
    SetPRCSwitch("PRC_FILE_END_des_treas_gold", 8);
    SetPRCSwitch("PRC_FILE_END_des_treas_items", 15);
    SetPRCSwitch("PRC_FILE_END_des_xp_rewards", 220);
    SetPRCSwitch("PRC_FILE_END_diffsettings", 6);
    SetPRCSwitch("PRC_FILE_END_disease", 62);
    SetPRCSwitch("PRC_FILE_END_dmgxp", 59);
    SetPRCSwitch("PRC_FILE_END_domains", 59);
    SetPRCSwitch("PRC_FILE_END_doortype", 2);
    SetPRCSwitch("PRC_FILE_END_doortypes", 238);
    SetPRCSwitch("PRC_FILE_END_ECL", 254);
    SetPRCSwitch("PRC_FILE_END_effectanim", 0);
    SetPRCSwitch("PRC_FILE_END_effecticons", 129);
    SetPRCSwitch("PRC_FILE_END_encdifficulty", 4);
    SetPRCSwitch("PRC_FILE_END_encumbrance", 255);
    SetPRCSwitch("PRC_FILE_END_environment", 25);
    SetPRCSwitch("PRC_FILE_END_epicattacks", 235);
    SetPRCSwitch("PRC_FILE_END_epicsaves", 59);
    SetPRCSwitch("PRC_FILE_END_epicspells", 70);
    SetPRCSwitch("PRC_FILE_END_epicspellseeds", 27);
    SetPRCSwitch("PRC_FILE_END_excitedduration", 2);
    SetPRCSwitch("PRC_FILE_END_exptable", 40);
    SetPRCSwitch("PRC_FILE_END_feat", 24439);
    SetPRCSwitch("PRC_FILE_END_fileends", 20);
    SetPRCSwitch("PRC_FILE_END_footstepsounds", 17);
    SetPRCSwitch("PRC_FILE_END_fractionalcr", 4);
    SetPRCSwitch("PRC_FILE_END_gamespyrooms", 12);
    SetPRCSwitch("PRC_FILE_END_gender", 4);
    SetPRCSwitch("PRC_FILE_END_genericdoors", 25);
    SetPRCSwitch("PRC_FILE_END_hen_companion", 18);
    SetPRCSwitch("PRC_FILE_END_hen_familiar", 12);
    SetPRCSwitch("PRC_FILE_END_inventorysnds", 32);
    SetPRCSwitch("PRC_FILE_END_iprp_abilities", 5);
    SetPRCSwitch("PRC_FILE_END_iprp_acmodtype", 4);
    SetPRCSwitch("PRC_FILE_END_iprp_addcost", 1);
    SetPRCSwitch("PRC_FILE_END_iprp_additional", 1);
    SetPRCSwitch("PRC_FILE_END_iprp_aligngrp", 5);
    SetPRCSwitch("PRC_FILE_END_iprp_alignment", 8);
    SetPRCSwitch("PRC_FILE_END_iprp_ammocost", 15);
    SetPRCSwitch("PRC_FILE_END_iprp_ammotype", 2);
    SetPRCSwitch("PRC_FILE_END_iprp_amount", 4);
    SetPRCSwitch("PRC_FILE_END_iprp_aoe", 7);
    SetPRCSwitch("PRC_FILE_END_iprp_arcspell", 19);
    SetPRCSwitch("PRC_FILE_END_iprp_base1", -1);
    SetPRCSwitch("PRC_FILE_END_iprp_bladecost", 5);
    SetPRCSwitch("PRC_FILE_END_iprp_bonuscost", 12);
    SetPRCSwitch("PRC_FILE_END_iprp_casterlvl", 60);
    SetPRCSwitch("PRC_FILE_END_iprp_chargecost", 13);
    SetPRCSwitch("PRC_FILE_END_iprp_color", 6);
    SetPRCSwitch("PRC_FILE_END_iprp_combatdam", 2);
    SetPRCSwitch("PRC_FILE_END_iprp_costtable", 39);
    SetPRCSwitch("PRC_FILE_END_iprp_damagecost", 70);
    SetPRCSwitch("PRC_FILE_END_iprp_damagetype", 13);
    SetPRCSwitch("PRC_FILE_END_iprp_damvulcost", 7);
    SetPRCSwitch("PRC_FILE_END_iprp_decvalue1", 9);
    SetPRCSwitch("PRC_FILE_END_iprp_decvalue2", 9);
    SetPRCSwitch("PRC_FILE_END_iprp_feats", 24819);
    SetPRCSwitch("PRC_FILE_END_iprp_immuncost", 7);
    SetPRCSwitch("PRC_FILE_END_iprp_immunity", 9);
    SetPRCSwitch("PRC_FILE_END_iprp_incvalue1", 9);
    SetPRCSwitch("PRC_FILE_END_iprp_incvalue2", 9);
    SetPRCSwitch("PRC_FILE_END_iprp_kitcost", 50);
    SetPRCSwitch("PRC_FILE_END_iprp_lightcost", 4);
    SetPRCSwitch("PRC_FILE_END_iprp_matcost", 77);
    SetPRCSwitch("PRC_FILE_END_iprp_material", 77);
    SetPRCSwitch("PRC_FILE_END_iprp_maxpp", 8);
    SetPRCSwitch("PRC_FILE_END_iprp_meleecost", 20);
    SetPRCSwitch("PRC_FILE_END_iprp_metamagic", 6);
    SetPRCSwitch("PRC_FILE_END_iprp_monstcost", 58);
    SetPRCSwitch("PRC_FILE_END_iprp_monsterdam", 14);
    SetPRCSwitch("PRC_FILE_END_iprp_monsterhit", 9);
    SetPRCSwitch("PRC_FILE_END_iprp_neg10cost", 50);
    SetPRCSwitch("PRC_FILE_END_iprp_neg5cost", 10);
    SetPRCSwitch("PRC_FILE_END_iprp_onhit", 25);
    SetPRCSwitch("PRC_FILE_END_iprp_onhitcost", 70);
    SetPRCSwitch("PRC_FILE_END_iprp_onhitdur", 27);
    SetPRCSwitch("PRC_FILE_END_iprp_onhitspell", 209);
    SetPRCSwitch("PRC_FILE_END_iprp_paramtable", 11);
    SetPRCSwitch("PRC_FILE_END_iprp_poison", 5);
    SetPRCSwitch("PRC_FILE_END_iprp_protection", 19);
    SetPRCSwitch("PRC_FILE_END_iprp_qualcost", 15);
    SetPRCSwitch("PRC_FILE_END_iprp_quality", 15);
    SetPRCSwitch("PRC_FILE_END_iprp_redcost", 5);
    SetPRCSwitch("PRC_FILE_END_iprp_resistcost", 28);
    SetPRCSwitch("PRC_FILE_END_iprp_saveelement", 21);
    SetPRCSwitch("PRC_FILE_END_iprp_savingthrow", 3);
    SetPRCSwitch("PRC_FILE_END_iprp_skillcost", 50);
    SetPRCSwitch("PRC_FILE_END_iprp_slotscost", -1);
    SetPRCSwitch("PRC_FILE_END_iprp_soakcost", 50);
    SetPRCSwitch("PRC_FILE_END_iprp_speed_dec", 9);
    SetPRCSwitch("PRC_FILE_END_iprp_speed_enh", 9);
    SetPRCSwitch("PRC_FILE_END_iprp_spellcost", 243);
    SetPRCSwitch("PRC_FILE_END_iprp_spellcstr", 42);
    SetPRCSwitch("PRC_FILE_END_iprp_spelllvcost", 9);
    SetPRCSwitch("PRC_FILE_END_iprp_spelllvlimm", 9);
    SetPRCSwitch("PRC_FILE_END_iprp_spells", 1456);
    SetPRCSwitch("PRC_FILE_END_iprp_spellshl", 7);
    SetPRCSwitch("PRC_FILE_END_iprp_srcost", 99);
    SetPRCSwitch("PRC_FILE_END_iprp_staminacost", -1);
    SetPRCSwitch("PRC_FILE_END_iprp_storedpp", 16);
    SetPRCSwitch("PRC_FILE_END_iprp_terraintype", -1);
    SetPRCSwitch("PRC_FILE_END_iprp_trapcost", 11);
    SetPRCSwitch("PRC_FILE_END_iprp_traps", 4);
    SetPRCSwitch("PRC_FILE_END_iprp_trapsize", 3);
    SetPRCSwitch("PRC_FILE_END_iprp_visualfx", 6);
    SetPRCSwitch("PRC_FILE_END_iprp_walk", 1);
    SetPRCSwitch("PRC_FILE_END_iprp_weightcost", 6);
    SetPRCSwitch("PRC_FILE_END_iprp_weightinc", 5);
    SetPRCSwitch("PRC_FILE_END_itempropdef", 199);
    SetPRCSwitch("PRC_FILE_END_itemprops", 199);
    SetPRCSwitch("PRC_FILE_END_itemvalue", 59);
    SetPRCSwitch("PRC_FILE_END_itmwizammo", 54);
    SetPRCSwitch("PRC_FILE_END_itmwizarmor", 72);
    SetPRCSwitch("PRC_FILE_END_itmwizhelmet", 71);
    SetPRCSwitch("PRC_FILE_END_itmwizmelee", 47);
    SetPRCSwitch("PRC_FILE_END_itmwizpotion", 38);
    SetPRCSwitch("PRC_FILE_END_itmwizranged", 46);
    SetPRCSwitch("PRC_FILE_END_itmwizrods", 41);
    SetPRCSwitch("PRC_FILE_END_itmwizscroll", 38);
    SetPRCSwitch("PRC_FILE_END_itmwizstaves", 60);
    SetPRCSwitch("PRC_FILE_END_itmwizthrow", 48);
    SetPRCSwitch("PRC_FILE_END_itmwiztrap", 43);
    SetPRCSwitch("PRC_FILE_END_itmwizwands", 38);
    SetPRCSwitch("PRC_FILE_END_keymap", 70);
    SetPRCSwitch("PRC_FILE_END_lightcolor", 32);
    SetPRCSwitch("PRC_FILE_END_loadhints", 88);
    SetPRCSwitch("PRC_FILE_END_loadscreens", 259);
    SetPRCSwitch("PRC_FILE_END_masterfeats", 113);
    SetPRCSwitch("PRC_FILE_END_materialcomp", 200);
    SetPRCSwitch("PRC_FILE_END_metamagic", 6);
    SetPRCSwitch("PRC_FILE_END_namefilter", 3);
    SetPRCSwitch("PRC_FILE_END_nwconfig", 6);
    SetPRCSwitch("PRC_FILE_END_nwconfig2", 6);
    SetPRCSwitch("PRC_FILE_END_packeqbarb1", 5);
    SetPRCSwitch("PRC_FILE_END_packeqbarb3", 7);
    SetPRCSwitch("PRC_FILE_END_packeqbarb4", 4);
    SetPRCSwitch("PRC_FILE_END_packeqbarb5", 5);
    SetPRCSwitch("PRC_FILE_END_packeqbard1", 11);
    SetPRCSwitch("PRC_FILE_END_packeqcler1", 6);
    SetPRCSwitch("PRC_FILE_END_packeqcler2", 6);
    SetPRCSwitch("PRC_FILE_END_packeqcler3", 6);
    SetPRCSwitch("PRC_FILE_END_packeqcler4", 6);
    SetPRCSwitch("PRC_FILE_END_packeqcler5", 6);
    SetPRCSwitch("PRC_FILE_END_packeqdruid1", 5);
    SetPRCSwitch("PRC_FILE_END_packeqfight1", 5);
    SetPRCSwitch("PRC_FILE_END_packeqfight2", 7);
    SetPRCSwitch("PRC_FILE_END_packeqfight6", 5);
    SetPRCSwitch("PRC_FILE_END_packeqfightc", 5);
    SetPRCSwitch("PRC_FILE_END_packeqmonk1", 5);
    SetPRCSwitch("PRC_FILE_END_packeqmonk2", 6);
    SetPRCSwitch("PRC_FILE_END_packeqmonk4", 5);
    SetPRCSwitch("PRC_FILE_END_packeqmonk5", 6);
    SetPRCSwitch("PRC_FILE_END_packeqpala1", 6);
    SetPRCSwitch("PRC_FILE_END_packeqpala2", 7);
    SetPRCSwitch("PRC_FILE_END_packeqpala3", 8);
    SetPRCSwitch("PRC_FILE_END_packeqrang1", 8);
    SetPRCSwitch("PRC_FILE_END_packeqrang2", 8);
    SetPRCSwitch("PRC_FILE_END_packeqrang3", 8);
    SetPRCSwitch("PRC_FILE_END_packeqrang4", 8);
    SetPRCSwitch("PRC_FILE_END_packeqrang5", 8);
    SetPRCSwitch("PRC_FILE_END_packeqrog1", 8);
    SetPRCSwitch("PRC_FILE_END_packeqrog2", 8);
    SetPRCSwitch("PRC_FILE_END_packeqrog3", 8);
    SetPRCSwitch("PRC_FILE_END_packeqrog5", 8);
    SetPRCSwitch("PRC_FILE_END_packeqrogd", 8);
    SetPRCSwitch("PRC_FILE_END_packeqsor1", 11);
    SetPRCSwitch("PRC_FILE_END_packeqwiz1", 11);
    SetPRCSwitch("PRC_FILE_END_packeqwizb", 11);
    SetPRCSwitch("PRC_FILE_END_packftarch", 168);
    SetPRCSwitch("PRC_FILE_END_packftassa", 220);
    SetPRCSwitch("PRC_FILE_END_packftbarb1", 222);
    SetPRCSwitch("PRC_FILE_END_packftbarb2", 219);
    SetPRCSwitch("PRC_FILE_END_packftbarb3", 220);
    SetPRCSwitch("PRC_FILE_END_packftbarb4", 222);
    SetPRCSwitch("PRC_FILE_END_packftbarb5", 221);
    SetPRCSwitch("PRC_FILE_END_packftbarbf", 37);
    SetPRCSwitch("PRC_FILE_END_packftbard1", 177);
    SetPRCSwitch("PRC_FILE_END_packftbard2", 173);
    SetPRCSwitch("PRC_FILE_END_packftbard3", 176);
    SetPRCSwitch("PRC_FILE_END_packftbard4", 169);
    SetPRCSwitch("PRC_FILE_END_packftbard5", 168);
    SetPRCSwitch("PRC_FILE_END_packftbard6", 67);
    SetPRCSwitch("PRC_FILE_END_packftbardg", 42);
    SetPRCSwitch("PRC_FILE_END_packftblck", 335);
    SetPRCSwitch("PRC_FILE_END_packftcler1", 161);
    SetPRCSwitch("PRC_FILE_END_packftcler2", 166);
    SetPRCSwitch("PRC_FILE_END_packftcler3", 165);
    SetPRCSwitch("PRC_FILE_END_packftcler4", 165);
    SetPRCSwitch("PRC_FILE_END_packftcler5", 163);
    SetPRCSwitch("PRC_FILE_END_packftcler6", 160);
    SetPRCSwitch("PRC_FILE_END_packftclere", 46);
    SetPRCSwitch("PRC_FILE_END_packftcrea1", 23);
    SetPRCSwitch("PRC_FILE_END_packftdrdis", 254);
    SetPRCSwitch("PRC_FILE_END_packftdruid1", 196);
    SetPRCSwitch("PRC_FILE_END_packftdruid2", 204);
    SetPRCSwitch("PRC_FILE_END_packftdruid3", 199);
    SetPRCSwitch("PRC_FILE_END_packftdruid4", 198);
    SetPRCSwitch("PRC_FILE_END_packftdruid5", 198);
    SetPRCSwitch("PRC_FILE_END_packftdruid6", 196);
    SetPRCSwitch("PRC_FILE_END_packftdwdef", 337);
    SetPRCSwitch("PRC_FILE_END_packftfight1", 382);
    SetPRCSwitch("PRC_FILE_END_packftfight2", 383);
    SetPRCSwitch("PRC_FILE_END_packftfight3", 383);
    SetPRCSwitch("PRC_FILE_END_packftfight4", 383);
    SetPRCSwitch("PRC_FILE_END_packftfight5", 382);
    SetPRCSwitch("PRC_FILE_END_packftfight6", 383);
    SetPRCSwitch("PRC_FILE_END_packftfightc", 45);
    SetPRCSwitch("PRC_FILE_END_packftharp", 191);
    SetPRCSwitch("PRC_FILE_END_packftmonk1", 275);
    SetPRCSwitch("PRC_FILE_END_packftmonk2", 275);
    SetPRCSwitch("PRC_FILE_END_packftmonk3", 275);
    SetPRCSwitch("PRC_FILE_END_packftmonk4", 275);
    SetPRCSwitch("PRC_FILE_END_packftmonk5", 275);
    SetPRCSwitch("PRC_FILE_END_packftmonk6", 275);
    SetPRCSwitch("PRC_FILE_END_packftpala1", 213);
    SetPRCSwitch("PRC_FILE_END_packftpala2", 213);
    SetPRCSwitch("PRC_FILE_END_packftpala3", 212);
    SetPRCSwitch("PRC_FILE_END_packftpala4", 212);
    SetPRCSwitch("PRC_FILE_END_packftpalah", 32);
    SetPRCSwitch("PRC_FILE_END_packftrang1", 155);
    SetPRCSwitch("PRC_FILE_END_packftrang2", 158);
    SetPRCSwitch("PRC_FILE_END_packftrang3", 158);
    SetPRCSwitch("PRC_FILE_END_packftrang4", 159);
    SetPRCSwitch("PRC_FILE_END_packftrang5", 158);
    SetPRCSwitch("PRC_FILE_END_packftrang6", 157);
    SetPRCSwitch("PRC_FILE_END_packftrog1", 298);
    SetPRCSwitch("PRC_FILE_END_packftrog2", 303);
    SetPRCSwitch("PRC_FILE_END_packftrog3", 302);
    SetPRCSwitch("PRC_FILE_END_packftrog5", 322);
    SetPRCSwitch("PRC_FILE_END_packftrog6", 315);
    SetPRCSwitch("PRC_FILE_END_packftrog7", 321);
    SetPRCSwitch("PRC_FILE_END_packftrogd", 37);
    SetPRCSwitch("PRC_FILE_END_packftshad", 287);
    SetPRCSwitch("PRC_FILE_END_packftshift", 148);
    SetPRCSwitch("PRC_FILE_END_packftsor1", 174);
    SetPRCSwitch("PRC_FILE_END_packftsor2", 157);
    SetPRCSwitch("PRC_FILE_END_packftsor3", 157);
    SetPRCSwitch("PRC_FILE_END_packftsor4", 157);
    SetPRCSwitch("PRC_FILE_END_packftsor5", 157);
    SetPRCSwitch("PRC_FILE_END_packftsor6", 172);
    SetPRCSwitch("PRC_FILE_END_packftsor7", 170);
    SetPRCSwitch("PRC_FILE_END_packftsor8", 174);
    SetPRCSwitch("PRC_FILE_END_packftsor9", 199);
    SetPRCSwitch("PRC_FILE_END_packftsora", 174);
    SetPRCSwitch("PRC_FILE_END_packfttorm", 360);
    SetPRCSwitch("PRC_FILE_END_packftwiz1", 174);
    SetPRCSwitch("PRC_FILE_END_packftwiz2", 157);
    SetPRCSwitch("PRC_FILE_END_packftwiz3", 157);
    SetPRCSwitch("PRC_FILE_END_packftwiz4", 157);
    SetPRCSwitch("PRC_FILE_END_packftwiz5", 157);
    SetPRCSwitch("PRC_FILE_END_packftwiz6", 172);
    SetPRCSwitch("PRC_FILE_END_packftwiz7", 170);
    SetPRCSwitch("PRC_FILE_END_packftwiz8", 176);
    SetPRCSwitch("PRC_FILE_END_packftwiz9", 199);
    SetPRCSwitch("PRC_FILE_END_packftwiza", 174);
    SetPRCSwitch("PRC_FILE_END_packftwizb", 43);
    SetPRCSwitch("PRC_FILE_END_packftwm", 259);
    SetPRCSwitch("PRC_FILE_END_packskarch", 10);
    SetPRCSwitch("PRC_FILE_END_packskassa", 13);
    SetPRCSwitch("PRC_FILE_END_packskbarb1", 20);
    SetPRCSwitch("PRC_FILE_END_packskbarb2", 20);
    SetPRCSwitch("PRC_FILE_END_packskbarb3", 20);
    SetPRCSwitch("PRC_FILE_END_packskbarb4", 20);
    SetPRCSwitch("PRC_FILE_END_packskbarb5", 20);
    SetPRCSwitch("PRC_FILE_END_packskbarb6", 20);
    SetPRCSwitch("PRC_FILE_END_packskbarb7", 8);
    SetPRCSwitch("PRC_FILE_END_packskbard1", 22);
    SetPRCSwitch("PRC_FILE_END_packskbard2", 22);
    SetPRCSwitch("PRC_FILE_END_packskbard3", 22);
    SetPRCSwitch("PRC_FILE_END_packskbard4", 22);
    SetPRCSwitch("PRC_FILE_END_packskbard5", 22);
    SetPRCSwitch("PRC_FILE_END_packskbard6", 22);
    SetPRCSwitch("PRC_FILE_END_packskbard7", 12);
    SetPRCSwitch("PRC_FILE_END_packskblck", 9);
    SetPRCSwitch("PRC_FILE_END_packskcler1", 22);
    SetPRCSwitch("PRC_FILE_END_packskcler2", 21);
    SetPRCSwitch("PRC_FILE_END_packskcler3", 21);
    SetPRCSwitch("PRC_FILE_END_packskcler4", 21);
    SetPRCSwitch("PRC_FILE_END_packskcler5", 4);
    SetPRCSwitch("PRC_FILE_END_packskcrea1", 4);
    SetPRCSwitch("PRC_FILE_END_packskdrdis", 10);
    SetPRCSwitch("PRC_FILE_END_packskdruid1", 21);
    SetPRCSwitch("PRC_FILE_END_packskdruid2", 21);
    SetPRCSwitch("PRC_FILE_END_packskdruid3", 21);
    SetPRCSwitch("PRC_FILE_END_packskdruid4", 21);
    SetPRCSwitch("PRC_FILE_END_packskdruid5", 22);
    SetPRCSwitch("PRC_FILE_END_packskdwdef", 9);
    SetPRCSwitch("PRC_FILE_END_packskfight1", 20);
    SetPRCSwitch("PRC_FILE_END_packskfight2", 20);
    SetPRCSwitch("PRC_FILE_END_packskfight3", 21);
    SetPRCSwitch("PRC_FILE_END_packskfight5", 20);
    SetPRCSwitch("PRC_FILE_END_packskfight6", 4);
    SetPRCSwitch("PRC_FILE_END_packskharp", 11);
    SetPRCSwitch("PRC_FILE_END_packskmonk1", 19);
    SetPRCSwitch("PRC_FILE_END_packskmonk6", 20);
    SetPRCSwitch("PRC_FILE_END_packskpala1", 18);
    SetPRCSwitch("PRC_FILE_END_packskpala4", 19);
    SetPRCSwitch("PRC_FILE_END_packskpalah", 21);
    SetPRCSwitch("PRC_FILE_END_packskrang1", 24);
    SetPRCSwitch("PRC_FILE_END_packskrang2", 24);
    SetPRCSwitch("PRC_FILE_END_packskrang3", 24);
    SetPRCSwitch("PRC_FILE_END_packskrog1", 23);
    SetPRCSwitch("PRC_FILE_END_packskrog2", 23);
    SetPRCSwitch("PRC_FILE_END_packskrog3", 23);
    SetPRCSwitch("PRC_FILE_END_packskrog4", 22);
    SetPRCSwitch("PRC_FILE_END_packskrog5", 21);
    SetPRCSwitch("PRC_FILE_END_packskrog6", 13);
    SetPRCSwitch("PRC_FILE_END_packskrog7", 23);
    SetPRCSwitch("PRC_FILE_END_packskshad", 11);
    SetPRCSwitch("PRC_FILE_END_packsksor10", 19);
    SetPRCSwitch("PRC_FILE_END_packsktorm", 23);
    SetPRCSwitch("PRC_FILE_END_packskwiz1", 19);
    SetPRCSwitch("PRC_FILE_END_packskwizb", 23);
    SetPRCSwitch("PRC_FILE_END_packspbar1", 67);
    SetPRCSwitch("PRC_FILE_END_packspbar2", 47);
    SetPRCSwitch("PRC_FILE_END_packspbar3", 34);
    SetPRCSwitch("PRC_FILE_END_packspcleric1", 56);
    SetPRCSwitch("PRC_FILE_END_packspcleric2", 53);
    SetPRCSwitch("PRC_FILE_END_packspdruid1", 56);
    SetPRCSwitch("PRC_FILE_END_packspnpc1", 101);
    SetPRCSwitch("PRC_FILE_END_packsppala1", 26);
    SetPRCSwitch("PRC_FILE_END_packsprang1", 63);
    SetPRCSwitch("PRC_FILE_END_packspwiz1", 146);
    SetPRCSwitch("PRC_FILE_END_packspwiz2", 47);
    SetPRCSwitch("PRC_FILE_END_packspwiz3", 47);
    SetPRCSwitch("PRC_FILE_END_packspwiz4", 50);
    SetPRCSwitch("PRC_FILE_END_packspwiz5", 48);
    SetPRCSwitch("PRC_FILE_END_packspwiz6", 47);
    SetPRCSwitch("PRC_FILE_END_packspwiz7", 47);
    SetPRCSwitch("PRC_FILE_END_packspwiz8", 49);
    SetPRCSwitch("PRC_FILE_END_packspwiz9", 44);
    SetPRCSwitch("PRC_FILE_END_packspwizb", 35);
    SetPRCSwitch("PRC_FILE_END_parts_belt", 200);
    SetPRCSwitch("PRC_FILE_END_parts_bicep", 200);
    SetPRCSwitch("PRC_FILE_END_parts_chest", 200);
    SetPRCSwitch("PRC_FILE_END_parts_foot", 200);
    SetPRCSwitch("PRC_FILE_END_parts_forearm", 200);
    SetPRCSwitch("PRC_FILE_END_parts_hand", 200);
    SetPRCSwitch("PRC_FILE_END_parts_legs", 200);
    SetPRCSwitch("PRC_FILE_END_parts_neck", 200);
    SetPRCSwitch("PRC_FILE_END_parts_pelvis", 200);
    SetPRCSwitch("PRC_FILE_END_parts_robe", 38);
    SetPRCSwitch("PRC_FILE_END_parts_shin", 200);
    SetPRCSwitch("PRC_FILE_END_parts_shoulder", 200);
    SetPRCSwitch("PRC_FILE_END_phenotype", 33);
    SetPRCSwitch("PRC_FILE_END_placeableobjsnds", 50);
    SetPRCSwitch("PRC_FILE_END_placeables", 846);
    SetPRCSwitch("PRC_FILE_END_placeabletypes", 8);
    SetPRCSwitch("PRC_FILE_END_poison", 146);
    SetPRCSwitch("PRC_FILE_END_poisontypedef", 3);
    SetPRCSwitch("PRC_FILE_END_polymorph", 155);
    SetPRCSwitch("PRC_FILE_END_portraits", 1300);
    SetPRCSwitch("PRC_FILE_END_prc_craft_alchem", 37);
    SetPRCSwitch("PRC_FILE_END_prc_craft_gen_it", 204);
    SetPRCSwitch("PRC_FILE_END_prc_craft_poison", 62);
    SetPRCSwitch("PRC_FILE_END_prc_domains", 59);
    SetPRCSwitch("PRC_FILE_END_prc_familiar", 10);
    SetPRCSwitch("PRC_FILE_END_prc_polymorph", 401);
    SetPRCSwitch("PRC_FILE_END_prc_rune_craft", 7);
    SetPRCSwitch("PRC_FILE_END_prc_spells", 4071);
    SetPRCSwitch("PRC_FILE_END_prc_weap_items", 58);
    SetPRCSwitch("PRC_FILE_END_pregen", 44);
    SetPRCSwitch("PRC_FILE_END_prioritygroups", 21);
    SetPRCSwitch("PRC_FILE_END_pvpsettings", 3);
    SetPRCSwitch("PRC_FILE_END_racialappear", 254);
    SetPRCSwitch("PRC_FILE_END_racialtypes", 254);
    SetPRCSwitch("PRC_FILE_END_ranges", 13);
    SetPRCSwitch("PRC_FILE_END_repadjust", 3);
    SetPRCSwitch("PRC_FILE_END_replacetexture", 1);
    SetPRCSwitch("PRC_FILE_END_repute", 4);
    SetPRCSwitch("PRC_FILE_END_resistancecost", -1);
    SetPRCSwitch("PRC_FILE_END_restduration", 60);
    SetPRCSwitch("PRC_FILE_END_rrf_nss", 19);
    SetPRCSwitch("PRC_FILE_END_rrf_wav", 40);
    SetPRCSwitch("PRC_FILE_END_shft_packages", 9);
    SetPRCSwitch("PRC_FILE_END_shft_pk_animal", 67);
    SetPRCSwitch("PRC_FILE_END_shft_pk_constrct", 15);
    SetPRCSwitch("PRC_FILE_END_shft_pk_dragon", 49);
    SetPRCSwitch("PRC_FILE_END_shft_pk_element", 15);
    SetPRCSwitch("PRC_FILE_END_shft_pk_gen1", 30);
    SetPRCSwitch("PRC_FILE_END_shft_pk_gen2", 43);
    SetPRCSwitch("PRC_FILE_END_shft_pk_outsider", 47);
    SetPRCSwitch("PRC_FILE_END_shft_pk_rogue", 66);
    SetPRCSwitch("PRC_FILE_END_shft_pk_undead", 43);
    SetPRCSwitch("PRC_FILE_END_shft_pk_warrior", 88);
    SetPRCSwitch("PRC_FILE_END_shifter_abilitie", 119);
    SetPRCSwitch("PRC_FILE_END_shifter_feats", 425);
    SetPRCSwitch("PRC_FILE_END_shifter_races", 30);
    SetPRCSwitch("PRC_FILE_END_skills", 38);
    SetPRCSwitch("PRC_FILE_END_skillvsitemcost", 55);
    SetPRCSwitch("PRC_FILE_END_skyboxes", 6);
    SetPRCSwitch("PRC_FILE_END_soundcatfilters", 14);
    SetPRCSwitch("PRC_FILE_END_sounddefaultspos", 3);
    SetPRCSwitch("PRC_FILE_END_sounddefaultstim", 5);
    SetPRCSwitch("PRC_FILE_END_soundeax", 112);
    SetPRCSwitch("PRC_FILE_END_soundgain", 14);
    SetPRCSwitch("PRC_FILE_END_soundprovider", 12);
    SetPRCSwitch("PRC_FILE_END_soundset", 453);
    SetPRCSwitch("PRC_FILE_END_soundsettype", 4);
    SetPRCSwitch("PRC_FILE_END_soundtypes", 1);
    SetPRCSwitch("PRC_FILE_END_spells", 19348);
    //SetPRCSwitch("PRC_FILE_END_spellschools", 9);
    SetPRCSwitch("PRC_FILE_END_statescripts", 35);
    SetPRCSwitch("PRC_FILE_END_stringtokens", 92);
    SetPRCSwitch("PRC_FILE_END_surfacemat", 30);
    SetPRCSwitch("PRC_FILE_END_swearfilter", 171);
    SetPRCSwitch("PRC_FILE_END_tailmodel", 572);
    SetPRCSwitch("PRC_FILE_END_tbw01_edge", 7);
    SetPRCSwitch("PRC_FILE_END_tcn01doors", 0);
    SetPRCSwitch("PRC_FILE_END_tcn01_edge", 27);
    SetPRCSwitch("PRC_FILE_END_tdc01_edge", 12);
    SetPRCSwitch("PRC_FILE_END_tde01_edge", 12);
    SetPRCSwitch("PRC_FILE_END_tdm01_edge", 13);
    SetPRCSwitch("PRC_FILE_END_tdr01_edge", 18);
    SetPRCSwitch("PRC_FILE_END_tds01_edge", 12);
    SetPRCSwitch("PRC_FILE_END_tdt01_edge", 11);
    SetPRCSwitch("PRC_FILE_END_templates", 105);
    SetPRCSwitch("PRC_FILE_END_tib01_edge", 3);
    SetPRCSwitch("PRC_FILE_END_tic01_edge", 18);
    SetPRCSwitch("PRC_FILE_END_tid01_edge", 10);
    SetPRCSwitch("PRC_FILE_END_tii01_edge", 6);
    SetPRCSwitch("PRC_FILE_END_tilecolor", 15);
    SetPRCSwitch("PRC_FILE_END_tin01doors", 0);
    SetPRCSwitch("PRC_FILE_END_tin01_edge", 15);
    SetPRCSwitch("PRC_FILE_END_tms01_edge", 6);
    SetPRCSwitch("PRC_FILE_END_tni01_edge", 15);
    SetPRCSwitch("PRC_FILE_END_tni02_edge", 19);
    SetPRCSwitch("PRC_FILE_END_tno01_edge", 67);
    SetPRCSwitch("PRC_FILE_END_traps", 101);
    SetPRCSwitch("PRC_FILE_END_treasurescale", 4);
    SetPRCSwitch("PRC_FILE_END_tsw01_edge", 12);
    SetPRCSwitch("PRC_FILE_END_ttd01_edge", 12);
    SetPRCSwitch("PRC_FILE_END_ttf01_edge", 12);
    SetPRCSwitch("PRC_FILE_END_tti01_edge", 10);
    SetPRCSwitch("PRC_FILE_END_ttr01doors", 0);
    SetPRCSwitch("PRC_FILE_END_ttr01_edge", 23);
    SetPRCSwitch("PRC_FILE_END_tts01doors", 2);
    SetPRCSwitch("PRC_FILE_END_tts01_edge", 25);
    SetPRCSwitch("PRC_FILE_END_ttu01_edge", 36);
    SetPRCSwitch("PRC_FILE_END_ttz01_edge", 28);
    SetPRCSwitch("PRC_FILE_END_twc03_edge", 12);
    SetPRCSwitch("PRC_FILE_END_unarmed_dmg", 13);
    SetPRCSwitch("PRC_FILE_END_vfx_fire_forget", 16);
    SetPRCSwitch("PRC_FILE_END_vfx_persistent", 254);
    SetPRCSwitch("PRC_FILE_END_videoquality", 9);
    SetPRCSwitch("PRC_FILE_END_visualeffects", 1373);
    SetPRCSwitch("PRC_FILE_END_waypoint", 4);
    SetPRCSwitch("PRC_FILE_END_weaponsounds", 21);
    SetPRCSwitch("PRC_FILE_END_wingmodel", 89);
    SetPRCSwitch("PRC_FILE_END_x3restrict", 899);
    SetPRCSwitch("PRC_FILE_END_xpbaseconst", 16);
    SetPRCSwitch("PRC_FILE_END_xptable", 39);
    //END AUTO-GENERATED FILEENDS


    //Overriding fileends
    SetPRCSwitch("PRC_FILE_END_feat", 390);
    SetPRCSwitch("PRC_FILE_END_spells", 539);
    //SetPRCSwitch("PRC_FILE_END_iprp_feats", 17993);
    SetPRCSwitch("PRC_FILE_END_iprp_feats", 40);
    //SetPRCSwitch("PRC_FILE_END_iprp_spells", 1379);


    //there is also the fileends.2da file, but that
    //isnt read in here yet. may be later though
    if(GetPRCSwitch(FILE_END_MANUAL))
        return;
    SetPRCSwitch(FILE_END_CLASSES,         PRCGetFileEnd("classes"));
    SetPRCSwitch(FILE_END_RACIALTYPES,     PRCGetFileEnd("racialtypes"));
    SetPRCSwitch(FILE_END_GENDER,          1);//overriden to 1 for convoCC m/f only choice
    SetPRCSwitch(FILE_END_PORTRAITS,       PRCGetFileEnd("portraits"));
    SetPRCSwitch(FILE_END_SKILLS,          PRCGetFileEnd("skills"));
    SetPRCSwitch(FILE_END_CLASS_FEAT,      600);
    SetPRCSwitch(FILE_END_CLASS_POWER,     300);
    SetPRCSwitch(FILE_END_CLASS_SPELLBOOK, PRCGetFileEnd("cls_spell_archv")); //
    SetPRCSwitch(FILE_END_FEAT,            PRCGetFileEnd("feat"));
    SetPRCSwitch(FILE_END_FAMILIAR,        PRCGetFileEnd("hen_familiar"));
    SetPRCSwitch(FILE_END_ANIMALCOMP,      PRCGetFileEnd("hen_companion"));
    SetPRCSwitch(FILE_END_DOMAINS,         PRCGetFileEnd("domains"));
    SetPRCSwitch(FILE_END_SOUNDSET,        PRCGetFileEnd("soundset"));
    SetPRCSwitch(FILE_END_SPELLS,          PRCGetFileEnd("spells"));
    //SetPRCSwitch(FILE_END_SPELLSCHOOL,     PRCGetFileEnd("spellschools"));
    SetPRCSwitch(FILE_END_APPEARANCE,      PRCGetFileEnd("appearance"));
    SetPRCSwitch(FILE_END_WINGS,           PRCGetFileEnd("wingmodel"));
    SetPRCSwitch(FILE_END_TAILS,           PRCGetFileEnd("tailmodel"));
    SetPRCSwitch(FILE_END_BASEITEMS,       PRCGetFileEnd("baseitems"));
}

void CreateSwitchNameArray()
{
    object oWP = GetWaypointByTag("PRC_Switch_Name_WP");
    if(!GetIsObjectValid(oWP))
        oWP = CreateObject(OBJECT_TYPE_WAYPOINT, "NW_WAYPOINT001", GetStartingLocation(), FALSE, "PRC_Switch_Name_WP");
    if(!GetIsObjectValid(oWP))
        PrintString("CreateSwitchNameArray: Problem creating waypoint.");
    array_create(oWP, "Switch_Name");
    //if you add more switches, add them to this list
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DEBUG);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_COMBAT_DEBUG);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_MATERIAL_COMPONENTS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DISABLE_COMPONENTS_SHOP);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_TRUESEEING);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_TRUESEEING_SPOT_BONUS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_BIOWARE_GRRESTORE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_BIOWARE_HEAL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_BIOWARE_MASS_HEAL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_BIOWARE_HARM);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_BIOWARE_NEUTRALIZE_POISON);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_BIOWARE_REMOVE_DISEASE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_TIMESTOP_BIOWARE_DURATION);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_TIMESTOP_LOCAL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_TIMESTOP_NO_HOSTILE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_TIMESTOP_BLANK_PC);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_AFTS_EXTRA_DAMAGE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_ELEMENTAL_SWARM);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_FEAR_AURAS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_TENSERS_TRANSFORMATION);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_BLACK_BLADE_OF_DISASTER);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_FIND_TRAPS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_DARKNESS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_DARKNESS_35ED);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_ANIMATE_DEAD);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CREATE_UNDEAD_PERMANENT);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CREATE_UNDEAD_UNCONTROLLED);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_NEC_TERM_PERMADEATH);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_SPELL_ALIGNMENT_RESTRICT);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_35ED_WORD_OF_FAITH);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_SLEEP_NO_HD_CAP);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_165_DEATH_IMMUNITY);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CASTERLEVEL_FIRST_CLASS_RULE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_USE_NEW_IMBUE_ARROW);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DRAGON_DISCIPLE_SIZE_CHANGES);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_SAMURAI_ALLOW_STOLEN_SACRIFICE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_SAMURAI_ALLOW_UNIDENTIFIED_SACRIFICE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_SAMURAI_SACRIFICE_SCALAR_x100);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_SAMURAI_VALUE_SCALAR_x100);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_ORC_WARLORD_COHORT);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_LICH_ALTER_SELF_DISABLE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_TRUE_NECROMANCER_ALTERNATE_VISUAL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_THRALLHERD_LEADERSHIP);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_EPIC_XP_COSTS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_EPIC_TAKE_TEN_RULE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_EPIC_PRIMARY_ABILITY_MODIFIER_RULE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_EPIC_BACKLASH_DAMAGE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_EPIC_FOCI_ADJUST_DC);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_EPIC_GOLD_MULTIPLIER);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_EPIC_XP_FRACTION);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_EPIC_FAILURE_FRACTION_GOLD);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_EPIC_BOOK_DESTRUCTION);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_UNIMPINGED);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_IMPENETRABILITY);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_DULLBLADES);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_CHAMPIONS_VALOR);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_EPIC_CONVO_LEARNING_DISABLE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_STAFF_CASTER_LEVEL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_NPC_HAS_PC_SPELLCASTING);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_ECL_USES_XP_NOT_HD);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DISABLE_DEMILICH);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_SPELLSLAB);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_SPELLSLAB_NOSCROLLS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_ABILITY_DAMAGE_EFFECTS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_SUPPLY_BASED_REST);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_REST_HEALING);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_REST_TIME);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_REST_LIMIT);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_SPELL_SCHOOLS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PLAYER_TIME);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_SOMATIC_COMPOMENTS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_SOMATIC_ITEMS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_MULTISUMMON);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_SUMMON_ROUND_PER_LEVEL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_ENABLE_SPELL_SHARING);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_ANIMAL_COMPANIONS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_FAMILIARS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_FAMILIAR_FEEDING);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_FAMILIARS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_NO_FREE_WIZ_SPELLS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_POWER_ATTACK);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_POWER_ATTACK_STACK_WITH_BW);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_NO_PETRIFY_GUI);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DISABLE_SWITCH_CHANGING_CONVO);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DISABLE_DOMAIN_ENFORCEMENT);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_BONUS_COHORTS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DISABLE_CUSTOM_COHORTS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DISABLE_STANDARD_COHORTS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DISABLE_REGISTER_COHORTS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_SLINGS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_BEBILITH_CLAWS_DESTROY);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_AUTO_IDENTIFY_ON_ACQUIRE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_AUTO_UNIDENTIFY_ON_UNACQUIRE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_SPELLFIRE_DISALLOW_CHARGE_SELF);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_SPELLFIRE_DISALLOW_DRAIN_SCROLL_POTION);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_SORC_DISALLOW_NEWSPELLBOOK);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_BARD_DISALLOW_NEWSPELLBOOK);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_BARD_LIGHT_ARMOR_SPELLCASTING);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CWSAMURAI_NO_HEIRLOOM_DAISHO);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DISABLE_CONVO_TEMPLATE_GAIN);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_ARMOR_SPEED);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_RACIAL_SPEED);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_FAST_TRAVEL_SPEED);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_REMOVE_PLAYER_SPEED);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_ENFORCE_RACIAL_APPEARANCE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_NPC_FORCE_RACE_ACQUIRE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DYNAMIC_CLOAK_AUTOCOLOUR_DISABLE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DEATH_STABLE_TO_DISABLED_CHANCE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DEATH_STABLE_TO_BLEED_CHANCE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DEATH_BLEED_TO_STABLE_CHANCE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DEATH_HEAL_FROM_STABLE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DEATH_DAMAGE_FROM_STABLE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DEATH_DAMAGE_FROM_BLEEDING);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DEATH_OR_BLEED);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_DEATH_ENABLE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_ACP_MANUAL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_ACP_AUTOMATIC);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_ACP_NPC_AUTOMATIC);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_ACP_DELAY);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_USE_TAGBASED_INDEX_FOR_POISON);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_USES_PER_ITEM_POISON_COUNT);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_USES_PER_ITEM_POISON_DIE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_ALLOW_ONLY_SHARP_WEAPONS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_ALLOW_ALL_POISONS_ON_WEAPONS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DEXBASED_WEAPON_POISONING_FAILURE_CHANCE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_USES_PER_WEAPON_POISON_COUNT);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_USES_PER_WEAPON_POISON_DIE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_POISON_ALLOW_CLEAN_IN_EQUIP);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CRAFT_POISON_USE_INGREDIENST);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PSI_ASTRAL_CONSTRUCT_USE_2DA);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PSI_ASTRAL_CONSTRUCT_DUR_MOD);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_RAPID_METABOLISM);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PSI_IMP_METAPSIONICS_USE_SUM);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PNP_SHFT_USECR);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PNP_SHFT_S_HUGE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PNP_SHFT_S_LARGE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PNP_SHFT_S_MEDIUM);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PNP_SHFT_S_SMALL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PNP_SHFT_S_TINY);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PNP_SHFT_F_OUTSIDER);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PNP_SHFT_F_ELEMENTAL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PNP_SHFT_F_CONSTRUCT);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PNP_SHFT_F_UNDEAD);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PNP_SHFT_F_DRAGON);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PNP_SHFT_F_ABERRATION);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PNP_SHFT_F_OOZE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PNP_SHFT_F_MAGICALBEAST);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PNP_SHFT_F_GIANT);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PNP_SHFT_F_VERMIN);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PNP_SHFT_F_BEAST);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PNP_SHFT_F_ANIMAL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PNP_SHFT_F_MONSTROUSHUMANOID);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PNP_SHFT_F_HUMANOID);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PNP_ELEMENTAL_DAMAGE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_SPELL_SNEAK_DISABLE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_3_5e_FIST_DAMAGE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_BRAWLER_SIZE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_APPEARANCE_SIZE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_BIOWARE_MONK_ATTACKS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_SMALL_CREATURE_FINESSE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_BIOWARE_DIVINE_POWER);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_ALLOW_SWITCH_OF_TARGET);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DISABLE_COUP_DE_GRACE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_FLAME_WEAPON_DAMAGE_MAX);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DARKFIRE_DAMAGE_MAX);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_TELEPORT_MAX_TARGET_LOCATIONS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DISABLE_TELEPORTATION);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PW_TIME);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PW_PC_AUTOEXPORT);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PW_HP_TRACKING);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PW_LOCATION_TRACKING);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PW_MAPPIN_TRACKING);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PW_DEATH_TRACKING);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PW_SPELL_TRACKING);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_XP_USE_BIOWARE_XPTABLE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_XP_USE_SIMPLE_LA);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_XP_USE_SIMPLE_RACIAL_HD);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_XP_USE_SIMPLE_RACIAL_HD_NO_FREE_XP);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_XP_USE_SIMPLE_RACIAL_HD_NO_SELECTION);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_XP_USE_SETXP);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_USE_DATABASE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_USE_BIOWARE_DATABASE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DB_PRECACHE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DB_SQLITE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DB_SQLITE_INTERVAL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DB_MYSQL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_USE_LETOSCRIPT);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_LETOSCRIPT_PHEONIX_SYNTAX);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_LETOSCRIPT_FIX_ABILITIES);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_LETOSCRIPT_UNICORN_SQL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_LETOSCRIPT_GETNEWESTBIC);

//craft
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_DISABLE_CRAFT);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CRAFT_TIMER_MULTIPLIER);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CRAFT_TIMER_MAX);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CRAFT_TIMER_MIN);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_BREW_POTION_CASTER_LEVEL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_SCRIBE_SCROLL_CASTER_LEVEL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CRAFT_WAND_CASTER_LEVEL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CRAFT_ROD_CASTER_LEVEL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CRAFT_STAFF_CASTER_LEVEL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CRAFTING_BASE_ITEMS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), X2_CI_BREWPOTION_MAXLEVEL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), X2_CI_BREWPOTION_COSTMODIFIER);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), X2_CI_SCRIBESCROLL_COSTMODIFIER);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), X2_CI_CRAFTWAND_MAXLEVEL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), X2_CI_CRAFTWAND_COSTMODIFIER);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CRAFTING_ARBITRARY);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CRAFTING_COST_SCALE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CRAFTING_TIME_SCALE);

//spells

//shifter

//general

//PW
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_ENABLE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_AVARIEL_WINGS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_FEYRI_WINGS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_AASIMAR_WINGS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_FEYRI_TAIL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_TIEFLING_TAIL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_DROW_ENFORCE_GENDER);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_GENASI_ENFORCE_DOMAINS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_RAKSHASA_FEMALE_APPEARANCE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_ENFORCE_PNP_RACIAL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_ENFORCE_FEATS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_DISALLOW_CUSTOMISE_WINGS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_DISALLOW_CUSTOMISE_TAIL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_DISALLOW_CUSTOMISE_MODEL);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_USE_RACIAL_APPEARANCES);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_USE_RACIAL_PORTRAIT);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_ONLY_PLAYER_VOICESETS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_RESTRICT_VOICESETS_BY_SEX);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_ALLOW_TO_KEEP_VOICESET);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_ALLOW_TO_KEEP_PORTRAIT);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_RESTRICT_PORTRAIT_BY_SEX);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_ENABLE_RACIAL_HITDICE);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_ALLOW_SKILL_POINT_ROLLOVER);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_USE_XP_FOR_NEW_CHAR);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_ENCRYPTION_KEY);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_STAT_POINTS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_BONUS_FEATS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_MAX_STAT);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_SKILL_MULTIPLIER);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_SKILL_BONUS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_CONVOCC_CUSTOM_EXIT_SCRIPT);

    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_TRUENAME_CR_MULTIPLIER);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_TRUENAME_LEVEL_BONUS);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_TRUENAME_DC_CONSTANT);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PERFECTED_MAP_CONSTANT);
    array_set_string(oWP, "Switch_Name", array_get_size(oWP, "Switch_Name"), PRC_PERFECTED_MAP_MULTIPLIER);
}
